/* LCD C-Based Demo Program for PJRC 8051 Development Board, Rev 4
 *   http://www.pjrc.com/tech/8051/board4/lcd_example.html
 *
 * This little demo prompts the user to enter characters on the
 * serial port and then displays them on the second line of the
 * LCD, while maintaining a count of the number of characters
 * on the first line.  Printf is used to write to both the LCD
 * and serial port using a custom putchar() function to easily
 * switch between them, so you can use printf's number formatting
 * and other features on both the LCD and serial port.
 *
 * Normally you would use the "lcd_driver.c" file in your own
 * project, and perhaps adapt the putchar() example from this
 * file to suit your application's use of printf (see the
 * comments below for using putchar)
 */


/* This code is an original work of authorship by PJRC.COM, LLC
 * and has been placed in the public domain.  That means there
 * is no copyright and you are free to use it however you like
 * in your own application, even in commercial for-profit products.
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * merchantability or fitness for a particular purpose.
 */


#include "lcd_driver.h"
#include "paulmon2.h"
#include <stdio.h>	/* for printf, printf_fast, etc */


/* This global variable determines if purchar (and printf) will
 * send text to the serial port or to the LCD.  If you have the
 * example purchar() code below, you can just set this bit to
 * 1 if you want printf() to display on the LCD, or to 0 if you
 * want printf() to send to the serial port.
 */

bit print_to_lcd=0;



/* A simple demo program to illustrate printing to the LCD
 * and also the serial port.  The comments make this code
 * appear lengthy, but there are only 14 non-comment lines
 * of code in the main loop!
 */

void main()
{
	unsigned int count=0;
	unsigned char x=0;
	char c;

	lcd_init();

	while (1) {
		/* first, clear the screen if we're starting */

		if (count == 0) lcd_clear();


		/* display # of chars in first line of LCD */

		lcd_home();
		print_to_lcd=1;
		printf_fast("# Char: %u", count);


		/* prompt the user to enter a char from the serial port */

		print_to_lcd=0;
		printf_fast("Type char #%u to copy to LCD:", count);


		/* get the char, and echo it nicely on the serial port */

		c = pm2_cin();
		if (c == 27) pm2_restart();  /* ESC to quit */
		printf_fast("%c\r\n", (char)c);
		   /* gotcha: using %c in printf requires explicit cast */
		   /* to type char, even if the variable already is a char! */


		/* now write the char to the LCD... notice that the  */
		/* lcd_XXXX functions always write to the LCD, regardless  */
		/* of the "print_to_lcd" variable, and the pm2_XXXX ones  */
		/* always send to the serial port.  If we wanted to use  */
		/* printf, then we'd need to change "print_to_lcd"  */

		lcd_set_xy(x, 1);
		lcd_putchar(c);


		/* increment the counts */

		x++;
		count++;
		if (x > 15) x = 0;
	}
}






/* printf may seem like a complicated beast, but all it really
 * does is take all the stuff you pass in and generate a bunch
 * of calls to putchar().  The printf functions in SDCC's library
 * do not actually "know" how to print anything at all!  They
 * require you to provide a putchar() function to do the real
 * work, one character at a time.
 * 
 * This simple but useful putchar() example uses a global variable
 * to switch where printf's output will go.  When (print_to_lcd == 1)
 * the characters will end up on the LCD, and when is it set to
 * zero, they will go to the serial port via PAULMON2's pm2_cout()
 * function.  If you have some other serial port driver, or some
 * other I/O device that printf() should be able to "print to",
 * all you need to do is adapt putchar() to route the stream of
 * characters where you want them.
 */

void putchar(char c)
{
	if (print_to_lcd) {
		lcd_putchar(c);
	} else {
		pm2_cout(c);
	}
}



